@@ -1,15 +1,12 @@
-#ifndef lint
-static char rcsId[]="$Header: /cvs/gnome/gnome-libs/gtk-xmhtml/StringUtil.c,v 1.4 1998/02/12 03:07:55 unammx Exp $";
-#endif
 /*****
 * StringUtil.c:  badly named file with misc. routines that don't fit anywhere
 *                else.
 *
-* This file Version	$Revision: 1.4 $
+* This file Version	$Revision: 1.5 $
 *
 * Creation date:		Wed May 29 22:35:32 GMT+0100 1996
-* Last modification: 	$Date: 1998/02/12 03:07:55 $
-* By:					$Author: unammx $
+* Last modification: 	$Date: 1999/07/29 01:26:28 $
+* By:					$Author: sopwith $
 * Current State:		$State: Exp $
 *
 * Author:				newt
@@ -41,6 +38,11 @@ static char rcsId[]="$Header: /cvs/gnome/gnome-libs/gtk-xmhtml/StringUtil.c,v 1.
 /*****
 * ChangeLog 
 * $Log: StringUtil.c,v $
+* Revision 1.5  1999/07/29 01:26:28  sopwith
+*
+*
+* Fix all warnings.
+*
 * Revision 1.4  1998/02/12 03:07:55  unammx
 * Merge to Koen's XmHTML 1.1.2 + following fixes:
 *
