@@ -1,14 +1,11 @@
-#ifndef lint
-static char rcsId[]="$Header: /debian/home/gnomecvs/gnome-libs/gtk-xmhtml/layout.c,v 1.5 1999/06/02 01:00:42 unammx Exp $";
-#endif
 /*****
 * layout.c : XmHTML layout computation routines
 *
-* This file Version	$Revision: 1.5 $
+* This file Version	$Revision: 1.6 $
 *
 * Creation date:		Thu Nov  6 01:35:46 GMT+0100 1997
-* Last modification: 	$Date: 1999/06/02 01:00:42 $
-* By:					$Author: unammx $
+* Last modification: 	$Date: 1999/07/29 01:26:29 $
+* By:					$Author: sopwith $
 * Current State:		$State: Exp $
 *
 * Author:				newt
@@ -36,6 +33,11 @@ static char rcsId[]="$Header: /debian/home/gnomecvs/gnome-libs/gtk-xmhtml/layout
 /*****
 * ChangeLog 
 * $Log: layout.c,v $
+* Revision 1.6  1999/07/29 01:26:29  sopwith
+*
+*
+* Fix all warnings.
+*
 * Revision 1.5  1999/06/02 01:00:42  unammx
 *
 * 1999-06-01  Akira Higuchi <a-higuti@math.sci.hokudai.ac.jp>
