@@ -1,14 +1,11 @@
-#ifndef lint
-static char rcsId[]="$Header: /debian/home/gnomecvs/gnome-libs/gtk-xmhtml/readJPEGplc.c,v 1.5 1998/02/12 03:09:47 unammx Exp $";
-#endif
 /*****
 * readJPEGplc.c :		JPEG progressive loading interfaces
 *
-* This file Version	$Revision: 1.5 $
+* This file Version	$Revision: 1.6 $
 *
 * Creation date:		Thu Jun 19 14:56:52 GMT+0100 1997
-* Last modification: 	$Date: 1998/02/12 03:09:47 $
-* By:					$Author: unammx $
+* Last modification: 	$Date: 1999/07/29 01:26:29 $
+* By:					$Author: sopwith $
 * Current State:		$State: Exp $
 *
 * Author:				newt
@@ -36,6 +33,11 @@ static char rcsId[]="$Header: /debian/home/gnomecvs/gnome-libs/gtk-xmhtml/readJP
 /*****
 * ChangeLog 
 * $Log: readJPEGplc.c,v $
+* Revision 1.6  1999/07/29 01:26:29  sopwith
+*
+*
+* Fix all warnings.
+*
 * Revision 1.5  1998/02/12 03:09:47  unammx
 * Merge to Koen's XmHTML 1.1.2 + following fixes:
 *
