@@ -52,6 +52,17 @@ char *alloca ();
 #include <ctype.h>
 #include <errno.h>
 
+#ifdef PREFER_DB1
+#ifdef HAVE_DB1_DB_H
+# include <db1/db.h>
+#else
+# ifdef HAVE_DB_185_H
+#  include <db_185.h>
+# else
+#  include <db.h>
+# endif
+#endif
+#else
 #ifdef HAVE_DB_185_H
 # include <db_185.h>
 #else
@@ -61,6 +72,7 @@ char *alloca ();
 #  include <db1/db.h>
 # endif
 #endif
+#endif
 
 #include "libgnomeP.h"
 #include "gnome-mime.h"
