@@ -48,6 +48,8 @@ typedef struct {
 	pid_t pid;
 	int fd;
 	int listen_fd;
+	int exit_status;	/* waitpid return value */
+	gboolean dead;		/* if the process is already dead  */
 } child_info_t;
 
 /*
@@ -59,7 +61,6 @@ static void
 sigchld_handler (int signo)
 {
 	GSList *l;
-	pid_t pid;
 	int status;
 
 	if (waitpid (helper_pid, &status, WNOHANG) == helper_pid){
@@ -71,6 +72,8 @@ sigchld_handler (int signo)
 		child_info_t *child = l->data;
 		
 		if (waitpid (child->pid, &status, WNOHANG) == child->pid){
+			child->exit_status = status;
+			child->dead = 1;
 			write (child->fd, "D", 1);
 			return;
 		}
@@ -131,7 +134,53 @@ s_pipe (int fd [2])
 	return socketpair (AF_UNIX, SOCK_STREAM, 0, fd);
 }
 
+#elif defined(__sgi) && !defined(HAVE_SENDMSG)
+
+/* 
+ * IRIX 6.2 is like 4.3BSD; it will not have HAVE_SENDMSG set, 
+ * because msghdr used msg_accrights and msg_accrightslen rather 
+ * than the newer msg_control and msg_controllen fields configure
+ * checks.  The SVR4 code below doesn't work because pipe()
+ * semantics are controlled by the svr3pipe systune variable, 
+ * which defaults to uni-directional pipes.  Also sending
+ * file descriptors through pipes isn't implemented.
+ */
+
+#include <sys/socket.h>
+#include <sys/uio.h>
+
+static int
+receive_fd (int helper_fd)
+{
+	struct iovec iov [1];
+	struct msghdr msg;
+	char buf [32];
+	int fd;
+
+	iov [0].iov_base = buf;
+	iov [0].iov_len  = sizeof (buf);
+	msg.msg_iov      = iov;
+	msg.msg_iovlen   = 1;
+	msg.msg_name     = NULL;
+	msg.msg_namelen  = 0;
+
+	msg.msg_accrights = (caddr_t) &fd;
+	msg.msg_accrightslen = sizeof(fd);
+
+	if (recvmsg (helper_fd, &msg, 0) <= 0)
+		return -1;
+
+	return fd;
+}
+
+static int
+s_pipe (int fd [2])
+{
+	return socketpair (AF_UNIX, SOCK_STREAM, 0, fd);
+}
+
 #else
+
 static int
 receive_fd (int helper_fd)
 {
@@ -204,8 +253,8 @@ get_ptys (int *master, int *slave, int update_wutmp)
 			 * (otherwise gnome-pty-heler wont notice when
 			 * this process is killed).
 			 */
-			fcntl (helper_socket_protocol [0], F_SETFD, 1);
-			fcntl (helper_socket_fdpassing [0], F_SETFD, 1);
+			fcntl (helper_socket_protocol [0], F_SETFD, FD_CLOEXEC);
+			fcntl (helper_socket_fdpassing [0], F_SETFD, FD_CLOEXEC);
 		}
 	}
 	op = GNOME_PTY_OPEN_NO_DB_UPDATE;
@@ -257,7 +306,6 @@ int
 zvt_init_subshell (struct vt_em *vt, char *pty_name, int log)
 {
 	int slave_pty, master_pty;
-	int subshell_pid;
 	struct sigaction sa;
 	child_info_t *child;
 	pid_t pid;
@@ -265,8 +313,15 @@ zvt_init_subshell (struct vt_em *vt, char *pty_name, int log)
 	int p[2];
 
 	g_return_val_if_fail (vt != NULL, -1);
-	
+
 	if (!sigchld_inited){
+		sigset_t sigset;
+
+		sigemptyset(&sigset);
+		sigaddset(&sigset, SIGPIPE);
+		sigaddset(&sigset, SIGCHLD);
+		sigprocmask(SIG_UNBLOCK, &sigset, NULL);
+
 		memset(&sa, 0, sizeof(sa));
 		sa.sa_handler = sigchld_handler;
 		sigaction (SIGCHLD, &sa, &old_sigchld_handler);
@@ -311,10 +366,12 @@ zvt_init_subshell (struct vt_em *vt, char *pty_name, int log)
 
 		vt->msgfd = p [0];
 		
-		child = g_malloc(sizeof(child_info_t));
+		child = g_new (child_info_t, 1);
 		child->pid = vt->childpid;
 		child->fd = p[1];
 		child->listen_fd = p [0];
+		child->dead = 0;
+		child->exit_status = 0;
 		children = g_slist_prepend (children, child);
 		
 		/* We could have received the SIGCHLD signal for the subshell 
@@ -361,7 +418,6 @@ zvt_shutdown_subshell (struct vt_em *vt)
 {
 	GnomePtyOps op;
 	GSList *l;
-	int status;
 
 	g_return_val_if_fail (vt != NULL, -1);
 
@@ -384,13 +440,19 @@ zvt_shutdown_subshell (struct vt_em *vt)
 		child_info_t *child = l->data;
 
 		if (child->pid == vt->childpid){
+			int status;
+			
+			if (!child->dead){
+				/* make sure the child has quit! */
+				kill (vt->childpid, SIGHUP);
+				waitpid (vt->childpid, &child->exit_status, 0);
+			}
+			
+			status = child->exit_status;
 			close (child->fd);
 			g_free (child);
 			children = g_slist_remove (children, l->data);
 
-			/* make sure the child has quit! */
-			kill(vt->childpid, SIGHUP);
-			waitpid(vt->childpid, &status, 0);
 			return WEXITSTATUS(status);
 		}
 	}
